Package: PhaseGMM
Type: Package
Title: Phase-Function Based Estimation and Inference for Linear
        Errors-in-Variables (EIV) Models
Version: 0.1.1
Authors@R: c(
    person(
      "Chang", "Liu",
      email = "leo12345liu@gmail.com",
      role = c("aut", "cre")
    ), person(
      "Linh", "Nghiem",
      email = "l.nghiemum@gmail.com",
      role = c("aut")
    )
  )
Description: Estimation and inference for coefficients of linear EIV models with symmetric measurement errors. 
      The measurement errors can be homoscedastic or heteroscedastic, for the latter, replication for at least some observations needs to be available. The estimation method and asymptotic inference are 
      based on a generalised method of moments framework, where the estimating equations are formed from (1) minimising the distance between the empirical phase function (normalised characteristic function) of the response and that of the linear combination of all the covariates at the estimates, 
      and (2) minimising a corrected least-square discrepancy function. Specifically, for a linear EIV model with p error-prone and q error-free covariates, if replicates are available, the GMM approach is based on a 2(p+q) estimating equations if some replicates are available and based on p+2q estimating equations if no replicate is available. 
      The details of the method are described in Nghiem and Potgieter (2020) <doi:10.1093/biomet/asaa025> and Nghiem and Potgieter (2025) <doi:10.5705/ss.202022.0331>.   
License: GPL-2
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
Depends: R (>= 3.5)
Imports: nleqslv
Suggests: extraDistr
NeedsCompilation: no
Packaged: 2026-04-28 10:38:38 UTC; liuchang
Author: Chang Liu [aut, cre],
  Linh Nghiem [aut]
Maintainer: Chang Liu <leo12345liu@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-28 11:00:02 UTC
Built: R 4.6.0; ; 2026-04-28 11:12:57 UTC; unix

Package: PhaseGMM
Type: Package
Title: Phase-Function Based Estimation and Inference for Linear
        Errors-in-Variables (EIV) Models
Version: 0.1.1
Authors@R: c(
    person(
      "Chang", "Liu",
      email = "leo12345liu@gmail.com",
      role = c("aut", "cre")
    ), person(
      "Linh", "Nghiem",
      email = "l.nghiemum@gmail.com",
      role = c("aut")
    )
  )
Description: Estimation and inference for coefficients of linear EIV models with symmetric measurement errors. 
      The measurement errors can be homoscedastic or heteroscedastic, for the latter, replication for at least some observations needs to be available. The estimation method and asymptotic inference are 
      based on a generalised method of moments framework, where the estimating equations are formed from (1) minimising the distance between the empirical phase function (normalised characteristic function) of the response and that of the linear combination of all the covariates at the estimates, 
      and (2) minimising a corrected least-square discrepancy function. Specifically, for a linear EIV model with p error-prone and q error-free covariates, if replicates are available, the GMM approach is based on a 2(p+q) estimating equations if some replicates are available and based on p+2q estimating equations if no replicate is available. 
      The details of the method are described in Nghiem and Potgieter (2020) <doi:10.1093/biomet/asaa025> and Nghiem and Potgieter (2025) <doi:10.5705/ss.202022.0331>.   
License: GPL-2
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
Depends: R (>= 3.5)
Imports: nleqslv
Suggests: extraDistr
NeedsCompilation: no
Packaged: 2026-04-28 10:38:38 UTC; liuchang
Author: Chang Liu [aut, cre],
  Linh Nghiem [aut]
Maintainer: Chang Liu <leo12345liu@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-28 11:00:02 UTC
Built: R 4.4.3; ; 2026-05-01 13:09:28 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(coef,eiv_mlr)
S3method(confint,eiv_mlr)
S3method(predict,eiv_mlr)
S3method(print,eiv_mlr)
S3method(residuals,eiv_mlr)
S3method(summary,eiv_mlr)
S3method(vcov,eiv_mlr)
export(eiv_mlr)
importFrom(stats,.getXlevels)
importFrom(stats,approx)
importFrom(stats,as.formula)
importFrom(stats,cov)
importFrom(stats,model.matrix)
importFrom(stats,optim)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,rbinom)
importFrom(stats,reformulate)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,symnum)
importFrom(stats,terms)
importFrom(stats,uniroot)

#' Print Method for eiv_mlr Objects
#'
#' Displays a concise summary of an errors-in-variables linear model fit.
#'
#' @param x An object of class \code{"eiv_mlr"}.
#' @param ... Not used.
#'
#' @return
#' Invisibly returns the input object \code{x} of class \code{"eiv_mlr"}.
#' This function is called for its side effect of printing a summary of the model.
#'
#' @export
print.eiv_mlr <- function(x, ...) {
  cat("\nErrors-in-Variables Linear Model\n")
  cat("Method:", x$method, "\n\n")
  cat("Coefficients:\n")
  print(x$coef)
  invisible(x)
}

#' Summary of an Errors-in-Variables Linear Model
#'
#' Produces a coefficient table including standard errors, z-statistics,
#' p-values, and significance stars.
#'
#' @param object An object of class \code{"eiv_mlr"}.
#' @param ... Not used.
#'
#' @return An object of class \code{"summary"} containing a
#'   coefficient table and model information.
#'
#' @export
summary.eiv_mlr <- function(object, ...) {

  zval <- object$coef / object$se
  pval <- 2 * pnorm(-abs(zval))

  stars <- symnum(
    pval,
    cutpoints = c(0, 0.001, 0.01, 0.05, 0.1, 1),
    symbols   = c("***", "**", "*", ".", "")
  )

  tab <- data.frame(
    Estimate = object$coef,
    Std.Err  = object$se,
    z.value  = zval,
    Pr.z     = pval,
    ` `      = as.character(stars),
    check.names = FALSE
  )

  res <- list(
    call   = object$call,
    method = object$method,
    table  = tab,
    n      = object$n
  )

  class(res) <- "summary.eiv_mlr"
  res
}


#' Extract Coefficients from an eiv_mlr Object
#'
#' @param object An object of class \code{"eiv_mlr"}.
#' @param ... Not used.
#'
#' @return A named numeric vector of estimated regression coefficients.
#'
#' @export

coef.eiv_mlr <- function(object, ...) {
  object$coef
}


#' Variance-Covariance Matrix for eiv_mlr Objects
#'
#' @param object An object of class \code{"eiv_mlr"}.
#' @param ... Not used.
#'
#' @return A numeric variance-covariance matrix of the parameter estimates.
#'
#' @export
vcov.eiv_mlr <- function(object, ...) {
  object$vcov
}


#' Confidence Intervals for eiv_mlr Coefficients
#'
#' Computes Wald-type confidence intervals for regression coefficients.
#'
#' @param object An object of class \code{"eiv_mlr"}.
#' @param parm Not used.
#' @param level Confidence level; defaults to 0.95.
#' @param ... Not used.
#'
#' @return A matrix with columns \code{lower} and \code{upper}.
#'
#' @export
confint.eiv_mlr <- function(object, parm = NULL, level = 0.95, ...) {

  alpha <- 1 - level
  z     <- qnorm(1 - alpha / 2)

  ci <- cbind(
    lower = object$coef - z * object$se,
    upper = object$coef + z * object$se
  )

  rownames(ci) <- names(object$coef)
  ci
}


#' Residuals from an Errors-in-Variables Linear Model
#'
#' Computes plug-in residuals based on observed covariates and estimated
#' coefficients. Measurement error is not corrected in residuals.
#'
#' @param object An object of class \code{"eiv_mlr"}.
#' @param ... Not used.
#'
#' @return A numeric vector of residuals.
#'
#' @export
residuals.eiv_mlr <- function(object, ...) {
  object$y - object$fitted
}


#' Predictions from an Errors-in-Variables Linear Model
#'
#' Generates fitted values or confidence intervals for new observations
#' using plug-in estimates. Measurement error is not corrected for new data.
#'
#' @param object An object of class \code{"eiv_mlr"}.
#' @param newdata Optional data frame containing covariates. If omitted,
#'   predictions are returned for the training data.
#' @param interval Type of interval to compute: \code{"none"} or
#'   \code{"confidence"}.
#' @param level Confidence level for intervals.
#' @param ... Not used.
#'
#' @return A numeric vector of predictions, or a matrix with columns
#'   \code{fit}, \code{lwr}, and \code{upr} if intervals are requested.
#'
#' @export


predict.eiv_mlr <- function(
    object,
    newdata = NULL,
    interval = c("none", "confidence"),
    level = 0.95,
    ...
) {

  interval <- match.arg(interval)

  if (is.null(newdata)) {
    return(object$fitted)
  }

  beta <- object$coef

  ## -----------------------
  ## 1. Intercept
  ## -----------------------
  intercept <- if ("(Intercept)" %in% names(beta)) {
    beta["(Intercept)"]
  } else {
    0
  }

  ## -----------------------
  ## 2. W part
  ## -----------------------
  beta_W <- beta[object$W_names]
  W_new  <- as.matrix(newdata[, object$W_names, drop = FALSE])

  W_part <- W_new %*% beta_W

  ## -----------------------
  ## 3. Z part
  ## -----------------------
  Zmat_new <- model.matrix(
    object$terms_Z,
    newdata,
    contrasts.arg = object$contrasts_Z,
    xlev = object$xlevels
  )

  beta_Z <- beta[colnames(Zmat_new)]
  Z_part <- Zmat_new %*% beta_Z

  ## -----------------------
  ## 4. Prediction
  ## -----------------------
  fit <- as.numeric(intercept + W_part + Z_part)

  if (interval == "none") {
    return(fit)
  }

  ## -----------------------
  ## 5. Confidence interval
  ## -----------------------
  V <- object$vcov
  beta_names <- names(beta)

  X_full <- matrix(0, nrow = nrow(W_new), ncol = length(beta_names))
  colnames(X_full) <- beta_names

  # Intercept
  if ("(Intercept)" %in% beta_names) {
    X_full[, "(Intercept)"] <- 1
  }

  # W variables
  for (nm in object$W_names) {
    if (nm %in% beta_names) {
      X_full[, nm] <- W_new[, nm]
    }
  }

  # Z variables
  for (nm in colnames(Zmat_new)) {
    if (nm %in% beta_names) {
      X_full[, nm] <- Zmat_new[, nm]
    }
  }

  se_fit <- sqrt(rowSums((X_full %*% V) * X_full))

  alpha <- 1 - level
  z <- qnorm(1 - alpha / 2)

  cbind(
    fit = fit,
    lwr = fit - z * se_fit,
    upr = fit + z * se_fit
  )
}

choose_beta_init <- function(
    y, Wbar, Zmat,
    S_y, C_y, t,
    estWeight,
    estSigmaU_bar
) {


  n  <- length(y)
  WZ <- cbind(Wbar, Zmat)
  p  <- ncol(Wbar)
  q  <- ncol(Zmat)

  ## Naive OLS estimator

  b_naive <- as.numeric(
    solve(crossprod(WZ), crossprod(WZ, y))
  )

  ## GMM1: identity-weighted Z + phase score

  f_gmm1 <- function(b) {

    stopifnot(length(b) == ncol(WZ))

    ## Phase score
    S_phase <- PhaseGradient(
      b, S_y, C_y,
      WZ, t,
      estWeight
    )

    S_corrected = estim_eq_corrected(y,
                                     Wbar,
                                     Zmat,
                                     b,
                                     estSigmaU_bar)

    ## Stack scores
    S <- c(
      as.vector(S_corrected),
      as.vector(S_phase)
    )

    ## Identity-weighted GMM objective
    sum(S^2)
  }

  opt <- optim(
    par     = b_naive,
    fn      = f_gmm1,
    method  = "Nelder-Mead",
    control = list(maxit = 10000)
  )

  if (opt$convergence != 0) {
    warning("GMM1 optimisation did not fully converge")
  }

  return(opt$par)
}

#' Estimate observation weights for measurement error models
#'
#' Computes observation-specific weights using uniform, minimax, and
#' quasi-likelihood weighting schemes based on replicate measurements of
#' contaminated covariates.
#'
#' @param W Numeric array of dimension \code{n x p x J} containing replicate
#'   measurements of contaminated covariates. Missing replicates should be
#'   coded as \code{NA}.
#' @param method Character string specifying which weights to return.
#'   One of \code{"all"}, \code{"uniform"}, \code{"minimax"},
#'   or \code{"quasi-likelihood"}.
#' @param gamma Optional non-negative regularisation parameter used in the
#'   quasi-likelihood weights. Defaults to \code{log(n)}.
#'
#' @return A numeric matrix with \code{n} rows. Columns correspond to the
#'   requested weighting schemes.
#'
#'
#'@keywords internal
estimating_weight <- function(
    W,
    method = c("uniform", "minimax", "quasi-likelihood"),
    gamma = NULL
) {
  n <- dim(W)[1]
  p <- dim(W)[2]
  J <- dim(W)[3]

  method <- match.arg(method)

  stopifnot(length(dim(W)) == 3)

  ## Early exit if no replicates
  has_rep <- apply(W[, 1, , drop = FALSE], 1, function(x) sum(!is.na(x)) > 1)

  ## Uniform weights
  w_uniform <- rep(1 / n, n)

  if (!any(has_rep)) {
    warning("No replicate information detected. Returning uniform weights.")
    return(matrix(w_uniform, ncol = 1, dimnames = list(NULL, "uniform")))
  }

  n <- dim(W)[1]
  p <- dim(W)[2]
  J <- dim(W)[3]

  if (is.null(gamma)) gamma <- log(n)

  ## Estimate Sigma_X
  Wbar <- apply(W, c(1, 2), mean, na.rm = TRUE)

  SigmaU_bar = estimating_SigmaU(W)[[1]]
  mean_SigmaU_bar = estimating_SigmaU(W)[[2]]

  SigmaX_hat <- cov(Wbar) - mean_SigmaU_bar

  ## Replace NA SigmaU_bar with mean_SigmaU_bar
  SigmaU_use <- SigmaU_bar

  for (i in seq_len(n)) {
    if (any(is.na(SigmaU_use[i, , ]))) {
      SigmaU_use[i, , ] <- mean_SigmaU_bar
    }
  }


  ## Minimax weights (Equation 3.4)

  lambda_max <- numeric(n)
  for (i in 1:n) {

    SigmaW_i <- SigmaX_hat + SigmaU_use[i, , ]
    lambda_max[i] <- max(eigen(SigmaW_i, symmetric = TRUE)$values)
  }

  w_minimax <- (1 / lambda_max)
  w_minimax <- w_minimax / sum(w_minimax)

  ## Quasi-likelihood weights

  A1 <- rep(0, p)
  A2 <- matrix(0, p, p)

  SigmaW <- array(0, dim = c(n, p, p))
  for (i in 1:n) {
    SigmaW[i, , ] <- SigmaX_hat + SigmaU_use[i, , ]
    Si_inv <- solve(SigmaW[i, , ])
    A1 <- A1 + as.numeric(Wbar[i, ] %*% Si_inv)
    A2 <- A2 + Si_inv
  }

  M1 <- rbind(
    cbind(
      Wbar %*% A2 %*% t(Wbar) +
        gamma * n * diag(n) -
        gamma * matrix(1, n, n),
      rep(1, n)
    ),
    c(rep(1, n), 0)
  )

  M2 <- c(Wbar %*% A1, 1)
  parms <- solve(M1, M2)

  w_ql <- parms[1:n]
  w_ql <- w_ql / sum(w_ql)

  ## Return

  if (method == "uniform") {
    return(matrix(w_uniform, ncol = 1, dimnames = list(NULL, "uniform")))
  }

  if (method == "minimax") {
    return(matrix(w_minimax, ncol = 1, dimnames = list(NULL, "minimax")))
  }

  if (method == "quasi-likelihood") {
    return(matrix(w_ql, ncol = 1, dimnames = list(NULL, "quasi_likelihood")))
  }
}

find_t_star <- function(y, n, t_max = 50, grid = 500) {

  f_root <- function(tt) {
    phi <- mean(exp(1i * tt * y))
    Re(phi * Conj(phi)) - 1 / sqrt(n)
  }

  t_grid <- seq(1e-4, t_max, length.out = grid)
  f_vals <- vapply(t_grid, f_root, numeric(1))

  idx <- which(diff(sign(f_vals)) != 0)

  if (length(idx) == 0) {
    return(max(t_grid))  # ← THE fallback
  }

  uniroot(
    f_root,
    lower = t_grid[idx[1]],
    upper = t_grid[idx[1] + 1]
  )$root
}

#' Linear Regression with Errors-in-Variables Using Replicated Measurements
#'
#' Fits a linear regression model in the presence of measurement error in
#' covariates using replicated measurements and a combination of
#' phase-function estimation and generalized method of moments (GMM).
#'
#' The function provides an \code{lm}-like interface while internally handling
#' replicated error-prone covariates, measurement error correction, and
#' robust variance estimation.
#'
#'
#'
#' @param formula A symbolic description of the model to be fitted.
#'   Error-prone covariates must be wrapped in \code{W()} and error-free
#'   covariates must be wrapped in \code{Z()}.
#'
#'   The general form is:
#'   \preformatted{
#'     y ~ W(W1 + W2 + ...) + Z(Z1 + Z2 + ...)
#'   }
#'
#'   An intercept is included automatically unless removed explicitly.
#'
#' @param data A data frame containing the response variable and all covariates
#'   appearing in \code{formula}. Each row corresponds to one replicate
#'   measurement of a statistical unit.
#'
#'   The data frame must contain:
#'   \itemize{
#'     \item A column named \code{unit} identifying statistical units.
#'     \item One or more rows per \code{unit} if replicated measurements exist.
#'     \item One column for each error-prone covariate (appearing in \code{W()}).
#'     \item One column for each error-free covariate (appearing in \code{Z()}).
#'   }
#'
#'   Replicated measurements are represented by multiple rows sharing the
#'   same \code{unit} identifier. Error-free covariates and the response
#'   should be constant within each \code{unit}.
#'
#' @param weight_method Character string specifying the observation weighting
#'   method used in estimation. One of:
#'   \describe{
#'     \item{\code{"uniform"}}{Uniform weights across observations.}
#'     \item{\code{"minimax"}}{Minimax optimal weights.}
#'     \item{\code{"quasi-likelihood"}}{Quasi-likelihood-based weights (default recommended).}
#'   }
#'
#' @param B Integer specifying the number of bootstrap replications used
#'   to estimate the GMM weighting matrix. Defaults to \code{100}.
#'
#' @param t_grid_length Integer specifying the number of frequency grid points
#'   used in phase-function integration. Larger values improve numerical
#'   accuracy at the cost of computation time.
#'
#' @details
#' This function implements a measurement-error-corrected linear regression
#' estimator for models with replicated error-prone covariates.
#' When fewer than two units contain replicated measurements, the function
#' automatically falls back to a quadratic (identity-weight) estimator.
#' This ensures the model remains estimable even in the absence of
#' replication.
#'
#' The estimation procedure:
#' \enumerate{
#'   \item Aggregates replicated measurements into a structured array.
#'   \item Uses phase-function estimating equations to correct for unknown
#'         measurement error distributions.
#'   \item Combines moment conditions via GMM when sufficient replication
#'         information is available.
#'   \item Automatically switches to a quadratic (identity-weight) estimator
#'         when fewer than two statistical units contain replicated measurements.
#' }
#'
#' Variance estimation is performed using a sandwich estimator, with the
#' GMM weighting matrix estimated via a cluster bootstrap over statistical
#' units.
#'
#' @return An object of class \code{"eiv_mlr"} containing:
#' \describe{
#'   \item{coef}{Estimated regression coefficients.}
#'   \item{vcov}{Estimated variance-covariance matrix.}
#'   \item{se}{Standard errors of the estimates.}
#'   \item{zvalue}{Z-statistics for hypothesis testing.}
#'   \item{pvalue}{Two-sided p-values.}
#'   \item{fitted}{Fitted values at the unit level.}
#'   \item{method}{Estimation method used (GMM or quadratic fallback).}
#'   \item{n}{Number of statistical units.}
#' }
#'
#' Standard methods such as \code{summary()}, \code{coef()},
#' \code{vcov()}, \code{confint()}, \code{predict()}, and
#' \code{residuals()} are available for objects of this class.
#'
#' @examples
#' ## ------------------------------------------
#' ## Small reproducible example (for speed reasons, we chose a too small number of bootstrap samples)
#' ## ------------------------------------------
#'
#' set.seed(1)
#'
#' n  <- 30
#' J  <- 2
#'
#' unit <- rep(1:n, each = J)
#'
#' W_true <- rnorm(n)
#' W_obs  <- rep(W_true, each = J) + rnorm(n * J, sd = 0.5)
#'
#' Z1 <- rep(rnorm(n), each = J)
#' y  <- rep(1 + 2 * W_true - 0.5 * Z1[seq(1, n * J, by = J)], each = J) +
#'       rnorm(n * J)
#'
#' sim_data <- data.frame(
#'   unit = unit,
#'   y = y,
#'   W1 = W_obs,
#'   Z1 = Z1
#' )
#'
#' # For speed reasons, we use a very small number of bootstrap samples
#' fit <- eiv_mlr(
#'   y ~ W(W1) + Z(Z1),
#'   data = sim_data,
#'   B = 10,
#'   t_grid_length = 20
#' )
#'
#' coef(fit)
#' summary(fit)
#'
#'
#' ## ------------------------------------------
#' ## Additional examples (not run during checks)
#' ## ------------------------------------------
#'
#' \donttest{
#' ## ------------------------------------------
#' ## Example using included dataset
#' ## ------------------------------------------
#'
#' fit <- eiv_mlr(
#'   bmi ~ W(energy + protein + fat) + Z(age_in_month),
#'   data = dietary_white_women,
#'   weight_method = "minimax",
#'   B = 100,
#'   t_grid_length = 200
#' )
#'
#' summary(fit)
#' confint(fit)
#'
#'
#' ## ------------------------------------------
#' ## Simulated example with replication
#' ## ------------------------------------------
#'
#' set.seed(1)
#'
#' n  <- 200
#' J  <- 2
#'
#' unit <- rep(1:n, each = J)
#'
#' W_true <- rnorm(n)
#' W_obs  <- rep(W_true, each = J) + rnorm(n * J, sd = 0.5)
#'
#' Z1 <- rep(rnorm(n), each = J)
#' y  <- rep(1 + 2 * W_true - 0.5 * Z1[seq(1, n * J, by = J)], each = J) +
#'       rnorm(n * J)
#'
#' sim_data <- data.frame(
#'   unit = unit,
#'   y = y,
#'   W1 = W_obs,
#'   Z1 = Z1
#' )
#'
#' fit_rep <- eiv_mlr(
#'   y ~ W(W1) + Z(Z1),
#'   data = sim_data,
#'   B = 20
#' )
#'
#' summary(fit_rep)
#'
#'
#' ## ------------------------------------------
#' ## Simulated example without replication
#' ## ------------------------------------------
#'
#' sim_norep <- sim_data[!duplicated(sim_data$unit), ]
#'
#' fit_norep <- eiv_mlr(
#'   y ~ W(W1) + Z(Z1),
#'   data = sim_norep,
#'   B = 20
#' )
#'
#' summary(fit_norep)
#' }
#'
#' @export



eiv_mlr <- function(
    formula,
    data,
    weight_method = c("uniform", "minimax", "quasi-likelihood"),
    B = 100,
    t_grid_length = 1000
) {

  weight_method <- match.arg(weight_method)

  ## Parse formula
  f <- parse_eiv_formula(formula)

  ## Prepare data
  prep <- prepare_eiv_data(
    data   = data,
    y_name = f$y,
    W_names = f$W,
    Z_names = f$Z
  )

  ## Core estimator
  fit <- computing_GMM_estimator(
    W = prep$W,
    y = prep$y,
    Zmat = prep$Zmat,
    weight_method = weight_method,
    B = B,
    t_grid_length = t_grid_length
  )

  est <- fit$est
  vc  <- fit$var_est

  ## Name coefficients
  coef_names <- c(colnames(prep$Wbar), colnames(prep$Zmat))
  names(est) <- coef_names
  dimnames(vc) <- list(coef_names, coef_names)

  ## Put intercept first
  if ("(Intercept)" %in% names(est)) {
    new_order <- c("(Intercept)", setdiff(names(est), "(Intercept)"))
    est <- est[new_order]
    vc  <- vc[new_order, new_order, drop = FALSE]
  }

  se  <- sqrt(diag(vc))
  zval <- est / se
  pval <- 2 * pnorm(-abs(zval))

  ## Fitted values
  beta_W <- est[colnames(prep$Wbar)]
  beta_Z <- est[colnames(prep$Zmat)]

  fitted <- as.numeric(
    prep$Wbar %*% beta_W +
      prep$Zmat %*% beta_Z
  )

  ## Store Z design info for prediction
  Z_formula <- reformulate(f$Z, intercept = TRUE)

  terms_Z  <- terms(Z_formula, data = data)
  xlevels  <- .getXlevels(terms_Z, data)

  contrasts_Z <- attr(prep$Zmat, "contrasts")

  out <- list(
    call    = match.call(),
    formula = formula,
    coef    = est,
    vcov    = vc,
    se      = se,
    zvalue  = zval,
    pvalue  = pval,
    method  = fit$method,
    n       = length(prep$y),
    y       = prep$y,
    fitted  = fitted,
    W_names = colnames(prep$Wbar),
    Z_names = colnames(prep$Zmat),

    terms_Z = terms_Z,
    contrasts_Z = contrasts_Z,
    xlevels = xlevels
  )

  class(out) <- "eiv_mlr"
  out
}

#' Combined estimating equations: moment correction and phase function
#'
#' Constructs a stacked system of estimating equations combining
#' moment-corrected score equations and phase-function score equations.
#'
#' @param Wbar Numeric matrix of dimension \code{n x p} containing averaged
#'   error-prone covariates.
#' @param Zmat Numeric matrix of dimension \code{n x q} containing error-free
#'   covariates (including an intercept if required).
#' @param y Numeric vector of length \code{n} containing the response.
#' @param S_y Numeric vector containing precomputed sine terms for the phase
#'   function.
#' @param C_y Numeric vector containing precomputed cosine terms for the phase
#'   function.
#' @param b_in Numeric vector of length \code{p + q} at which the estimating
#'   equations are evaluated.
#' @param estWeight Numeric vector of weights used in the phase-function
#'   component.
#' @param estSigmaU_bar Numeric array of dimension \code{n x p x p} containing
#'   measurement error covariance estimates.
#' @param t Numeric vector of frequency values used in the phase function.
#'
#' @return Numeric vector containing the stacked estimating equations of length
#'   \code{2 * (p + q)}.
#'
#'
#'@keywords internal

estim_eq_correctedAndPhase <- function(
    Wbar, Zmat, y, S_y, C_y, b_in, estWeight, estSigmaU_bar, t
) {
  # Wbar          : n × p averaged error-prone covariates
  # Zmat          : n × q error-free covariates (incl. intercept)
  # y             : n × 1 response
  # estSigmaU_bar : n × p × p (NA for single-replicate obs)

  n <- length(y)
  p <- ncol(Wbar)
  q <- ncol(Zmat)

  ## Moment-corrected score function
  Score_moment = estim_eq_corrected(y, Wbar, Zmat, b_in, estSigmaU_bar)

  ## phase-function score
  S_phase <- PhaseGradient(b_in, S_y, C_y, cbind(Wbar, Zmat), t, estWeight)

  ## Stack equations
  E <- c(
    as.vector(Score_moment),
    as.vector(S_phase)
  )

  return(E)
}

#' Weighting function for the phase–function estimator
#'
#' Computes the weighting function \eqn{K(t)} used within the
#' \code{computeD()} objective for the phase–function estimator.
#' The weight controls how much influence each frequency value receives
#' when integrating the squared phase–difference between \eqn{Y}
#' and the linear predictor \eqn{Wb}.
#'
#' The user may choose between several polynomial or absolute–value kernels
#' (via \code{type}), or a Gaussian kernel (via \code{delta}). If
#' \code{delta} is supplied, the Gaussian weight is used regardless of
#' \code{type}.
#'
#' @param t Numeric vector of frequency values, usually normalised to lie
#'   in \eqn{[0, 1]}.
#' @param type Character string specifying the weighting kernel:
#'   \itemize{
#'     \item \code{"quadratic"}: \eqn{(1 - t)^2}
#'     \item \code{"triangular"}: \eqn{1 - |t|}
#'     \item \code{"sqrt"}: \eqn{1 - \sqrt{t}}
#'     \item \code{"parabolic"}: \eqn{1 - t^2}
#'   }
#'   Ignored if \code{delta} is supplied.
#'
#' @param delta Optional numeric value. When supplied, a Gaussian weight
#'   \eqn{\exp(-\delta t^2 / 2)} is used. This may offer improved numerical
#'   stability for noisy empirical characteristic functions.
#'
#' @returns A numeric vector of weights of the same length as \code{t}.
#'
#' @noRd

omega <- function(t, type = c("quadratic", "triangular", "sqrt", "parabolic"),
                  delta = NULL) {

  # Gaussian kernel overrides other choices
  if (!is.null(delta)) {
    return( exp(-0.5 * delta * t^2) )
  }

  # Match type safely
  type <- match.arg(type)

  if (type == "quadratic") {
    return((1 - t)^2)

  } else if (type == "triangular") {
    return(1 - abs(t))

  } else if (type == "sqrt") {
    return(1 - sqrt(t))

  } else if (type == "parabolic") {
    return(1 - t^2)
  }
}

compute_P1_moment <- function(Wbar, Zmat, meanSigmaU) {

  n <- nrow(Wbar)
  p <- ncol(Wbar)
  q <- ncol(Zmat)
  P_zero <- matrix(0, nrow = q, ncol = p)

  # Univariate case
  if (is.null(dim(meanSigmaU))) {

    # β–γ
    P_bg <- -sqrt(n) * crossprod(Wbar, Zmat) / n
    # γ–β
    P_gb <- -sqrt(n) * crossprod(Zmat, Wbar) / n
    # γ–γ
    P_gg <- -sqrt(n) * crossprod(Zmat) / n

    # β–β
    if (is.na(meanSigmaU)) {

      return(cbind(P_zero, P_gg))
    } else {
      P_bb <- sqrt(n) * (
        -crossprod(Wbar) / n + matrix(meanSigmaU, 1, 1)
      )
    }

  } else {

    if (any(is.na(meanSigmaU))) {
      # no replicate information
      # # β–β
      # P_bb <- sqrt(n) * ( -crossprod(Wbar) / n )
      # # β–γ
      # P_bg <- -sqrt(n) * crossprod(Wbar, Zmat) / n
      # # γ–β
      # P_gb <- -sqrt(n) * crossprod(Zmat, Wbar) / n
      # γ–γ
      P_gg <- -sqrt(n) * crossprod(Zmat) / n
      return(cbind(P_zero, P_gg))

    } else {
      # β–β
      P_bb <- sqrt(n) * ( -crossprod(Wbar) / n + meanSigmaU )
      # β–γ
      P_bg <- -sqrt(n) * crossprod(Wbar, Zmat) / n
      # γ–β
      P_gb <- -sqrt(n) * crossprod(Zmat, Wbar) / n
      # γ–γ
      P_gg <- -sqrt(n) * crossprod(Zmat) / n
    }

  }

  rbind(
    cbind(P_bb, P_bg),
    cbind(P_gb, P_gg)
  )
}

#' dietary_white_women
#'
#' A processed Dataset containing repeated 24-hour dietary recalls.
#'
#' @format A data frame with 2 rows per individual (one per recall day).
#' \describe{
#'   \item{unit}{Integer unit identifier}
#'   \item{bmi}{Body Mass Index (kg/m^2)}
#'   \item{energy}{Total energy intake (kcal)}
#'   \item{protein}{Protein intake (g)}
#'   \item{fat}{Fat intake (g)}
#'   \item{age_in_month}{Age in months}
#'   \item{replicate}{Replicate id of each observation}
#' }
#'
#' @details
#' The variables \code{energy}, \code{protein}, and \code{fat}
#' are treated as error-prone covariates with two replicate
#' measurements per individual, and age_in_month is treated as error-free
#' covariate.
#'
#' @source Processed from NHANES dietary recall data.
"dietary_white_women"

W <- function(...) {
  vars <- as.character(substitute(list(...)))[-1]
  structure(vars, class = "W_terms")
}

Z <- function(...) {
  vars <- as.character(substitute(list(...)))[-1]
  structure(vars, class = "Z_terms")
}

extract_vars <- function(expr) {
  if (is.symbol(expr)) {
    return(as.character(expr))
  }
  if (is.call(expr) && as.character(expr[[1]]) == "+") {
    return(c(
      extract_vars(expr[[2]]),
      extract_vars(expr[[3]])
    ))
  }
  stop("Unsupported expression inside W() or Z()")
}


parse_eiv_formula <- function(formula) {

  f <- as.formula(formula)
  lhs <- as.character(f[[2]])
  rhs <- f[[3]]

  if (!is.call(rhs)) {
    stop("Right-hand side must be a formula")
  }

  W_vars <- character(0)
  Z_vars <- character(0)

  ## Walk through RHS terms
  terms <- if (rhs[[1]] == "+") as.list(rhs)[-1] else list(rhs)

  for (term in terms) {

    if (is.call(term) && term[[1]] == as.name("W")) {
      W_vars <- extract_vars(term[[2]])
    }

    if (is.call(term) && term[[1]] == as.name("Z")) {
      Z_vars <- extract_vars(term[[2]])
    }
  }

  if (length(W_vars) == 0) {
    stop("No W(...) term found in formula")
  }

  list(
    y = lhs,
    W = W_vars,
    Z = Z_vars
  )
}

prepare_eiv_data <- function(data, y_name, W_names, Z_names) {

  if (!"unit" %in% names(data)) {
    stop("data must contain a column named 'unit' identifying statistical units")
  }

  units <- unique(data$unit)
  n     <- length(units)
  p     <- length(W_names)
  J     <- max(table(data$unit))

  ## Build W array: n × p × J
  W <- array(NA_real_, dim = c(n, p, J),
             dimnames = list(units, W_names, NULL))

  for (i in seq_along(units)) {
    rows <- data$unit == units[i]
    Wi   <- as.matrix(data[rows, W_names])
    W[i, , seq_len(nrow(Wi))] <- t(Wi)
  }

  ## One row per unit
  data_u <- data[!duplicated(data$unit), ]

  y <- data_u[[y_name]]

  if (length(Z_names) > 0) {

    Z_formula <- reformulate(Z_names, intercept = TRUE)

    terms_Z   <- terms(Z_formula, data = data_u)

    Zmat <- model.matrix(terms_Z, data_u)

  } else {

    Zmat <- matrix(1, nrow = nrow(data_u), ncol = 1)

    colnames(Zmat) <- "(Intercept)"

    terms_Z <- NULL

  }

  Wbar <- apply(W, c(1, 2), mean, na.rm = TRUE)

  list(
    y = y,
    Zmat = Zmat,
    W = W,
    Wbar = Wbar,
    terms_Z = terms_Z,
    contrasts_Z = attr(Zmat, "contrasts")
  )
}

#' Moment-corrected estimating equations for measurement error regression
#'
#' Constructs the moment-corrected estimating equations for a linear regression
#' model with averaged error-prone covariates and error-free covariates.
#'
#' @param y Numeric vector of length \code{n} containing the response.
#' @param Wbar Numeric matrix of dimension \code{n x p} containing averaged
#'   error-prone covariates.
#' @param Zmat Numeric matrix of dimension \code{n x q} containing error-free
#'   covariates (including an intercept if required).
#' @param b_in Numeric vector of length \code{p + q} containing the regression
#'   coefficients at which the estimating equations are evaluated.
#' @param estSigmaU_bar Numeric array of dimension \code{n x p x p} (or numeric
#'   vector in the univariate case) containing measurement error covariance
#'   estimates.
#'
#' @return Numeric vector of length \code{p + q} containing the stacked
#'   estimating equations.
#'
#'@keywords internal

estim_eq_corrected <- function(y, Wbar, Zmat, b_in, estSigmaU_bar) {

  n <- length(y)
  p <- ncol(Wbar)

  beta  <- b_in[1:p]
  gamma <- b_in[-(1:p)]

  r <- y - Wbar %*% beta - Zmat %*% gamma

  ## Score for Z
  SZ <- (t(Zmat) %*% r) / sqrt(n)

  ## Moment-corrected score for W

  # Univariate case
  if (is.null(dim(estSigmaU_bar)) || dim(estSigmaU_bar)[2] == 1) {

    meanSigmaU <- mean(estSigmaU_bar, na.rm = TRUE)

    if (is.na(meanSigmaU)) {
      # no replicate information → naive score
      return(SZ)
    } else {
      SW <- (t(Wbar) %*% r + n * meanSigmaU * beta) / sqrt(n)
    }

  } else {
    # Multivariate case
    meanSigmaU <- apply(estSigmaU_bar, c(2, 3), mean, na.rm = TRUE)

    if (any(is.na(meanSigmaU))) {
      # no replicate information → naive score
      return(SZ)
    } else {
      SW <- (t(Wbar) %*% r + n * meanSigmaU %*% beta) / sqrt(n)
    }
  }

  ## Stack equations
  rbind(SW, SZ)
}

#' GMM estimator combining moment-corrected and phase-function equations
#'
#' Computes Generalized Method of Moments (GMM) estimators by combining
#' moment-corrected estimating equations and phase-function estimating
#' equations. The GMM weighting matrix is estimated via a cluster bootstrap
#' that accounts for uncertainty in estimating the measurement error
#' covariance.
#'
#' @param W Numeric array of dimension \code{n x p x J} containing replicate
#'   measurements of contaminated covariates.
#' @param y Numeric vector of length \code{n} containing the response.
#' @param Zmat Numeric matrix of dimension \code{n x q} containing error-free
#'   covariates.
#' @param B Integer specifying the number of bootstrap replicates.
#' @param t_grid_length Integer specifying the length of optimal t for
#' computational intergration.
#'
#'
#' @return A list with components:
#' \describe{
#'   \item{est}{Numeric matrix of GMM estimates; one column per weight choice.}
#'   \item{CV_boot}{Array of bootstrap covariance matrices used in GMM.}
#' }
#'
#'@keywords internal
computing_GMM_estimator <- function(
    W, y, Zmat,
    weight_method = c("uniform", "minimax", "quasi-likelihood"),
    B = 100,
    t_grid_length = 1000
) {

  ## Detect replicate observations

  has_rep <- apply(
    W[, 1, , drop = FALSE],
    1,
    function(x) sum(!is.na(x)) > 1
  )

  ind_rep    <- which(has_rep)
  ind_nonrep <- which(!has_rep)


  # number of obs having replicates
  n_rep <- sum(has_rep)

  if (n_rep == 0) {

    warning("No replicate information detected. Returning uniform weights.")

  }

  # weight for estimation
  weight_method <- match.arg(weight_method)


  Wbar <- apply(W, c(1, 2), mean, na.rm = TRUE)

  n <- length(y)
  p <- ncol(Wbar)
  J <- dim(W)[3]
  gamma <- log(n)

  ## Original weight
  # w_origin = estimating_weight(
  #   W,
  #   method = weight_method,
  #   gamma = gamma
  # )[, 1]

  w_origin = suppressWarnings(
    estimating_weight(
      W,
      method = weight_method,
      gamma = gamma
    )
  )[, 1]

  t_star <- find_t_star(y, n)
  t <- seq(0.001 * t_star, t_star, length.out = t_grid_length)


  ## Original Sy, Cy
  tvec_origin <- outer(y, t)
  S_y <- colSums(sin(tvec_origin))
  C_y <- colSums(cos(tvec_origin))


  ## Estimate Sigma_U
  sigma_U_bar <- estimating_SigmaU(W)[[1]]


  ## Choose initial beta

  b_in <- choose_beta_init(
      y             = y,
      Wbar          = Wbar,
      Zmat          = Zmat,
      S_y           = S_y,
      C_y           = C_y,
      t             = t,
      estWeight     = w_origin,
      estSigmaU_bar = sigma_U_bar
    )

  q <- length(b_in)

  ## Storage for bootstrap estimating equations

  g0 <- estim_eq_correctedAndPhase(
    Wbar, Zmat, y,
    S_y, C_y,
    b_in,
    w_origin,
    sigma_U_bar,
    t
  )

  E <- matrix(NA_real_, nrow = B, ncol = length(g0))

  # E <- matrix(NA_real_, nrow = B, ncol = 2*q)

  ## Bootstrap stage: estimate Omega

  for (b in seq_len(B)) {

    ## Cluster bootstrap
    ind_b <- c(
      sample(ind_rep,    length(ind_rep),    replace = TRUE),
      sample(ind_nonrep, length(ind_nonrep), replace = TRUE)
    )

    ## Bootstrap data
    Wb     <- W[ind_b, , , drop = FALSE]
    Wbar_b <- apply(Wb, c(1, 2), mean, na.rm = TRUE)
    yb     <- y[ind_b]
    Zb     <- Zmat[ind_b, , drop = FALSE]

    ## Re-estimate Sigma_U
    sigma_U_bar_b <- estimating_SigmaU(Wb)[[1]]

    ## Observation weights (computed internally)
    # w_b <- estimating_weight(
    #   Wb,
    #   method = weight_method,
    #   gamma = gamma
    # )[, 1]

    w_b <- suppressWarnings(
      estimating_weight(
        Wb,
        method = weight_method,
        gamma = gamma
      )
    )[, 1]

    ## Phase-function quantities

    t_star_b <- find_t_star(yb, n)
    t_b <- seq(0.001 * t_star_b, t_star_b, length.out = 1000)

    tvec <- outer(yb, t_b)

    S_y_b <- colSums(sin(tvec))
    C_y_b <- colSums(cos(tvec))

    ## Evaluate estimating equations

    E[b, ] <- estim_eq_correctedAndPhase(
      Wbar_b, Zb, yb,
      S_y_b, C_y_b,
      b_in,
      w_b,
      sigma_U_bar_b,
      t_b
    )
  }

  ## GMM stage

  Omega_hat <- cov(E)

  ## CASE 1: < 2 replicates -> QUADRATIC estimator
  if (n_rep < 2) {

    warning(
      "Fewer than two observations with replicates detected. ",
      "Using quadratic (identity-weight) estimator."
    )

    w_uniform <- rep(1 / n, n)

    est = b_in

    ## Jacobian
    P_moment <- compute_P1_moment(
      Wbar,
      Zmat,
      apply(sigma_U_bar, c(2, 3), mean, na.rm = TRUE)
    )

    P_phase <- compute_P1_phase(
      b = est,
      S_y = S_y,
      C_y = C_y,
      Wbar = Wbar,
      Zmat = Zmat,
      t = t,
      estWeight = w_uniform
    )

    P <- rbind(P_moment, P_phase)

    ## Sandwich variance (identity-weight GMM)
    PtP_inv <- solve(t(P) %*% P)
    var_est <- PtP_inv %*% t(P) %*% Omega_hat %*% P %*% PtP_inv

    return(list(
      est     = est,
      var_est = var_est,
      method  = "quadratic (identity-weight)"
    ))
  }


  ## CASE 2: ≥ 2 replicates -> TWO-STEP GMM

  obj <- function(b) {
    g <- estim_eq_correctedAndPhase(
      Wbar, Zmat, y,
      S_y, C_y,
      b,
      w_origin,
      sigma_U_bar,
      t
    )
    as.numeric(t(g) %*% solve(Omega_hat) %*% g)
  }

  est <- optim(b_in, obj, method = "Nelder-Mead", control = list(maxit = 10000))$par

  P_moment <- compute_P1_moment(
    Wbar,
    Zmat,
    apply(sigma_U_bar, c(2,3), mean, na.rm = TRUE)
  )

  P_phase <- compute_P1_phase(
    b = est,
    S_y = S_y,
    C_y = C_y,
    Wbar = Wbar,
    Zmat = Zmat,
    t = t,
    estWeight = w_origin
  )

  P <- rbind(
    P_moment,
    P_phase
  )

  Omega_inv <- solve(Omega_hat)

  var_est <- solve(
    t(P) %*% Omega_inv %*% P
  )


  return(list(
    est     = est,
    var_est = var_est,
    method  = "Generalised Method of Moments"
  ))
}

#' Gradient of the phase-function estimating equations
#'
#' Computes the gradient of the phase-function estimating equations with
#' respect to the regression coefficients.
#'
#' @param beta Numeric vector of length \code{p + q} containing the regression
#'   coefficients.
#' @param S_y Numeric vector containing \code{sum_j sin(t * Y_j)} evaluated on
#'   the grid \code{t}.
#' @param C_y Numeric vector containing \code{sum_j cos(t * Y_j)} evaluated on
#'   the grid \code{t}.
#' @param WZ Numeric matrix of dimension \code{n x (p + q)} containing all
#'   covariates.
#' @param t Numeric vector giving the grid of frequency values.
#' @param estWeight Numeric vector of length \code{n} giving observation
#'   weights.
#'
#' @return Numeric vector of length \code{p + q} containing the phase-function
#'   gradient evaluated at \code{beta}.
#'
#'
#'
#'
#'@keywords internal

PhaseGradient <- function(beta, S_y, C_y, WZ, t, estWeight) {
  # beta      : (p+q) × 1 coefficient vector
  # S_y       : vector of sum_j sin(t Y_j), length = length(t)
  # C_y       : vector of sum_j cos(t Y_j), length = length(t)
  # WZ        : n × (p+q) matrix of covariates
  # t         : grid of t values
  # estWeight : length-n vector of observation weights

  n <- nrow(WZ)
  p_tot <- length(beta)

  tDist <- t[2] - t[1]

  # kernel weights
  kernel <- omega(t / max(t), type = "quadratic")

  # trig arguments
  tvec <- matrix(t, nrow = n, ncol = length(t), byrow = TRUE)
  linpred <- as.vector(WZ %*% beta)
  trigArg <- linpred * tvec

  S <- sin(trigArg)
  C <- cos(trigArg)

  # weights
  estWeight <- as.vector(estWeight)
  Q <- matrix(estWeight, nrow = n, ncol = length(t))

  QC <- Q * C
  QS <- Q * S

  grad <- numeric(p_tot)

  # Gradient computation
  for (dimIndex in seq_len(p_tot)) {

    dimFactor <- WZ[, dimIndex] * tvec

    outer <- colSums(QC) * S_y - colSums(QS) * C_y

    inner <- -(colSums(dimFactor * QS) * S_y) -
      (colSums(dimFactor * QC) * C_y)

    integrand <- outer * inner * kernel * tDist

    grad[dimIndex] <- 2 * sum(integrand)
  }

  grad / n
}

compute_P1_phase <- function(
    b, S_y, C_y, Wbar, Zmat, t, estWeight, h = 1e-6
) {
  p_tot <- length(b)
  P <- matrix(0, p_tot, p_tot)

  base <- PhaseGradient(
    b, S_y, C_y, cbind(Wbar, Zmat), t, estWeight
  )

  for (j in seq_len(p_tot)) {
    b_up   <- b
    b_down <- b

    b_up[j]   <- b_up[j] + h
    b_down[j] <- b_down[j] - h

    g_up <- PhaseGradient(
      b_up, S_y, C_y, cbind(Wbar, Zmat), t, estWeight
    )

    g_down <- PhaseGradient(
      b_down, S_y, C_y, cbind(Wbar, Zmat), t, estWeight
    )

    P[, j] <- (g_up - g_down) / (2 * h)
  }

  P
}

#' Compute the corrected estimator via estimating equations
#'
#' This function computes the corrected estimator for a linear model
#' with measurement error in covariates by solving the corrected
#' estimating equations using a nonlinear root-finding algorithm.
#'
#' @param y Numeric vector of length \code{n}. Response variable.
#' @param Wbar Numeric matrix of dimension \code{n × p}. Error-prone
#'   covariates at the averaged (replicate) level.
#' @param Zmat Numeric matrix of dimension \code{n × q}. Error-free
#'   covariates (including intercept if applicable).
#' @param estSigmaU_bar Measurement error variance/covariance associated
#'   with the averaged covariates. Can be \code{n × p × p} or \code{n × 1}.
#' @param beta_in Optional numeric vector of length \code{p + q}. Initial
#'   value for the estimating equation solver.
#'
#' @return Numeric vector of length \code{p + q}, the corrected estimator.
#'
#' @details
#' The estimator is obtained by solving
#' \deqn{E_n(\beta) = 0}
#' where \code{E_n} is the corrected estimating equation implemented in
#' \code{estim_eq_corrected()}.
#'
#' @seealso \code{\link{estim_eq_corrected}}
#'
#'@keywords internal
#'
computing_corrected_estimator <- function(y,
                                          Wbar,
                                          Zmat,
                                          estSigmaU_bar,
                                          beta_in = NULL) {

  n <- length(y)
  p <- ncol(Wbar)
  q <- ncol(Zmat)

  # default initial value
  if (is.null(beta_in)) {
    beta_in <- rnorm(p + q)
  }

  # estimating equation
  D <- function(b) {
    estim_eq_corrected(y, Wbar, Zmat, b, estSigmaU_bar)
  }

  # solve D(b) = 0
  sol <- nleqslv::nleqslv(
    x = beta_in,
    fn = D,
    control = list(trace = 0)
  )

  sol$x
}

#' Estimate measurement error covariance matrices
#'
#' Computes observation-specific estimates of the measurement error
#' covariance matrix using replicate measurements of contaminated covariates.
#'
#' @param W Numeric array of dimension \code{n x p x J} containing replicate
#'   measurements of contaminated covariates. Missing replicates should be
#'   coded as \code{NA}.
#'
#' @return Numeric array of dimension \code{n x p x p}. If an observation has
#'   fewer than two available replicates, the corresponding covariance matrix
#'   is returned as \code{NA}.
#'
#' @noRd

estimating_SigmaU <- function(W) {

  stopifnot(length(dim(W)) == 3)

  n <- dim(W)[1]
  p <- dim(W)[2]
  Jmax <- dim(W)[3]

  SigmaU_hat <- array(NA, dim = c(n, p, p))

  for (i in 1:n) {

    # find valid replicates for obs i
    valid <- which(!is.na(W[i, 1, ]))
    ni <- length(valid)

    if (ni < 2) {
      SigmaU_hat[i, , ] <- NA
      next
    }

    acc <- matrix(0, p, p)

    for (a in 1:(ni - 1)) {
      for (b in (a + 1):ni) {

        r1 <- valid[a]
        r2 <- valid[b]

        diff <- W[i, , r1] - W[i, , r2]
        acc <- acc + tcrossprod(diff)
      }
    }

    SigmaU_hat[i, , ] <- acc / (ni * (ni - 1))
  }

  ## Sigma_u_bar
  Wbar <- apply(W, c(1, 2), mean, na.rm = TRUE)
  nj <- apply(W[, 1, , drop = FALSE], 1, function(x) sum(!is.na(x)))

  ## Sigma_U after averaging
  SigmaU_bar <- array(0, dim = c(n, p, p))
  for (i in 1:n) {
    SigmaU_bar[i, , ] <- SigmaU_hat[i, , ] / nj[i]
  }

  mean_SigmaU_bar <- apply(SigmaU_bar, c(2, 3), mean, na.rm = TRUE)

  list(SigmaU_bar, mean_SigmaU_bar)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Wrapper_output_functions.R
\name{confint.eiv_mlr}
\alias{confint.eiv_mlr}
\title{Confidence Intervals for eiv_mlr Coefficients}
\usage{
\method{confint}{eiv_mlr}(object, parm = NULL, level = 0.95, ...)
}
\arguments{
\item{object}{An object of class \code{"eiv_mlr"}.}

\item{parm}{Not used.}

\item{level}{Confidence level; defaults to 0.95.}

\item{...}{Not used.}
}
\value{
A matrix with columns \code{lower} and \code{upper}.
}
\description{
Computes Wald-type confidence intervals for regression coefficients.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Wrapper_output_functions.R
\name{residuals.eiv_mlr}
\alias{residuals.eiv_mlr}
\title{Residuals from an Errors-in-Variables Linear Model}
\usage{
\method{residuals}{eiv_mlr}(object, ...)
}
\arguments{
\item{object}{An object of class \code{"eiv_mlr"}.}

\item{...}{Not used.}
}
\value{
A numeric vector of residuals.
}
\description{
Computes plug-in residuals based on observed covariates and estimated
coefficients. Measurement error is not corrected in residuals.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Wrapper_output_functions.R
\name{summary.eiv_mlr}
\alias{summary.eiv_mlr}
\title{Summary of an Errors-in-Variables Linear Model}
\usage{
\method{summary}{eiv_mlr}(object, ...)
}
\arguments{
\item{object}{An object of class \code{"eiv_mlr"}.}

\item{...}{Not used.}
}
\value{
An object of class \code{"summary"} containing a
  coefficient table and model information.
}
\description{
Produces a coefficient table including standard errors, z-statistics,
p-values, and significance stars.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estim_eq_corrected.R
\name{estim_eq_corrected}
\alias{estim_eq_corrected}
\title{Moment-corrected estimating equations for measurement error regression}
\usage{
estim_eq_corrected(y, Wbar, Zmat, b_in, estSigmaU_bar)
}
\arguments{
\item{y}{Numeric vector of length \code{n} containing the response.}

\item{Wbar}{Numeric matrix of dimension \code{n x p} containing averaged
error-prone covariates.}

\item{Zmat}{Numeric matrix of dimension \code{n x q} containing error-free
covariates (including an intercept if required).}

\item{b_in}{Numeric vector of length \code{p + q} containing the regression
coefficients at which the estimating equations are evaluated.}

\item{estSigmaU_bar}{Numeric array of dimension \code{n x p x p} (or numeric
vector in the univariate case) containing measurement error covariance
estimates.}
}
\value{
Numeric vector of length \code{p + q} containing the stacked
  estimating equations.
}
\description{
Constructs the moment-corrected estimating equations for a linear regression
model with averaged error-prone covariates and error-free covariates.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimatingweight.R
\name{estimating_weight}
\alias{estimating_weight}
\title{Estimate observation weights for measurement error models}
\usage{
estimating_weight(
  W,
  method = c("uniform", "minimax", "quasi-likelihood"),
  gamma = NULL
)
}
\arguments{
\item{W}{Numeric array of dimension \code{n x p x J} containing replicate
measurements of contaminated covariates. Missing replicates should be
coded as \code{NA}.}

\item{method}{Character string specifying which weights to return.
One of \code{"all"}, \code{"uniform"}, \code{"minimax"},
or \code{"quasi-likelihood"}.}

\item{gamma}{Optional non-negative regularisation parameter used in the
quasi-likelihood weights. Defaults to \code{log(n)}.}
}
\value{
A numeric matrix with \code{n} rows. Columns correspond to the
  requested weighting schemes.
}
\description{
Computes observation-specific weights using uniform, minimax, and
quasi-likelihood weighting schemes based on replicate measurements of
contaminated covariates.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Wrapper_EIV.R
\name{eiv_mlr}
\alias{eiv_mlr}
\title{Linear Regression with Errors-in-Variables Using Replicated Measurements}
\usage{
eiv_mlr(
  formula,
  data,
  weight_method = c("uniform", "minimax", "quasi-likelihood"),
  B = 100,
  t_grid_length = 1000
)
}
\arguments{
\item{formula}{A symbolic description of the model to be fitted.
  Error-prone covariates must be wrapped in \code{W()} and error-free
  covariates must be wrapped in \code{Z()}.

  The general form is:
  \preformatted{
    y ~ W(W1 + W2 + ...) + Z(Z1 + Z2 + ...)
  }

  An intercept is included automatically unless removed explicitly.}

\item{data}{A data frame containing the response variable and all covariates
  appearing in \code{formula}. Each row corresponds to one replicate
  measurement of a statistical unit.

  The data frame must contain:
  \itemize{
    \item A column named \code{unit} identifying statistical units.
    \item One or more rows per \code{unit} if replicated measurements exist.
    \item One column for each error-prone covariate (appearing in \code{W()}).
    \item One column for each error-free covariate (appearing in \code{Z()}).
  }

  Replicated measurements are represented by multiple rows sharing the
  same \code{unit} identifier. Error-free covariates and the response
  should be constant within each \code{unit}.}

\item{weight_method}{Character string specifying the observation weighting
method used in estimation. One of:
\describe{
  \item{\code{"uniform"}}{Uniform weights across observations.}
  \item{\code{"minimax"}}{Minimax optimal weights.}
  \item{\code{"quasi-likelihood"}}{Quasi-likelihood-based weights (default recommended).}
}}

\item{B}{Integer specifying the number of bootstrap replications used
to estimate the GMM weighting matrix. Defaults to \code{100}.}

\item{t_grid_length}{Integer specifying the number of frequency grid points
used in phase-function integration. Larger values improve numerical
accuracy at the cost of computation time.}
}
\value{
An object of class \code{"eiv_mlr"} containing:
\describe{
  \item{coef}{Estimated regression coefficients.}
  \item{vcov}{Estimated variance-covariance matrix.}
  \item{se}{Standard errors of the estimates.}
  \item{zvalue}{Z-statistics for hypothesis testing.}
  \item{pvalue}{Two-sided p-values.}
  \item{fitted}{Fitted values at the unit level.}
  \item{method}{Estimation method used (GMM or quadratic fallback).}
  \item{n}{Number of statistical units.}
}

Standard methods such as \code{summary()}, \code{coef()},
\code{vcov()}, \code{confint()}, \code{predict()}, and
\code{residuals()} are available for objects of this class.
}
\description{
Fits a linear regression model in the presence of measurement error in
covariates using replicated measurements and a combination of
phase-function estimation and generalized method of moments (GMM).
}
\details{
The function provides an \code{lm}-like interface while internally handling
replicated error-prone covariates, measurement error correction, and
robust variance estimation.


This function implements a measurement-error-corrected linear regression
estimator for models with replicated error-prone covariates.
When fewer than two units contain replicated measurements, the function
automatically falls back to a quadratic (identity-weight) estimator.
This ensures the model remains estimable even in the absence of
replication.

The estimation procedure:
\enumerate{
  \item Aggregates replicated measurements into a structured array.
  \item Uses phase-function estimating equations to correct for unknown
        measurement error distributions.
  \item Combines moment conditions via GMM when sufficient replication
        information is available.
  \item Automatically switches to a quadratic (identity-weight) estimator
        when fewer than two statistical units contain replicated measurements.
}

Variance estimation is performed using a sandwich estimator, with the
GMM weighting matrix estimated via a cluster bootstrap over statistical
units.
}
\examples{
## ------------------------------------------
## Small reproducible example (for speed reasons, we chose a too small number of bootstrap samples)
## ------------------------------------------

set.seed(1)

n  <- 30
J  <- 2

unit <- rep(1:n, each = J)

W_true <- rnorm(n)
W_obs  <- rep(W_true, each = J) + rnorm(n * J, sd = 0.5)

Z1 <- rep(rnorm(n), each = J)
y  <- rep(1 + 2 * W_true - 0.5 * Z1[seq(1, n * J, by = J)], each = J) +
      rnorm(n * J)

sim_data <- data.frame(
  unit = unit,
  y = y,
  W1 = W_obs,
  Z1 = Z1
)

# For speed reasons, we use a very small number of bootstrap samples
fit <- eiv_mlr(
  y ~ W(W1) + Z(Z1),
  data = sim_data,
  B = 10,
  t_grid_length = 20
)

coef(fit)
summary(fit)


## ------------------------------------------
## Additional examples (not run during checks)
## ------------------------------------------

\donttest{
## ------------------------------------------
## Example using included dataset
## ------------------------------------------

fit <- eiv_mlr(
  bmi ~ W(energy + protein + fat) + Z(age_in_month),
  data = dietary_white_women,
  weight_method = "minimax",
  B = 100,
  t_grid_length = 200
)

summary(fit)
confint(fit)


## ------------------------------------------
## Simulated example with replication
## ------------------------------------------

set.seed(1)

n  <- 200
J  <- 2

unit <- rep(1:n, each = J)

W_true <- rnorm(n)
W_obs  <- rep(W_true, each = J) + rnorm(n * J, sd = 0.5)

Z1 <- rep(rnorm(n), each = J)
y  <- rep(1 + 2 * W_true - 0.5 * Z1[seq(1, n * J, by = J)], each = J) +
      rnorm(n * J)

sim_data <- data.frame(
  unit = unit,
  y = y,
  W1 = W_obs,
  Z1 = Z1
)

fit_rep <- eiv_mlr(
  y ~ W(W1) + Z(Z1),
  data = sim_data,
  B = 20
)

summary(fit_rep)


## ------------------------------------------
## Simulated example without replication
## ------------------------------------------

sim_norep <- sim_data[!duplicated(sim_data$unit), ]

fit_norep <- eiv_mlr(
  y ~ W(W1) + Z(Z1),
  data = sim_norep,
  B = 20
)

summary(fit_norep)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computing_GMM_estimator.R
\name{computing_GMM_estimator}
\alias{computing_GMM_estimator}
\title{GMM estimator combining moment-corrected and phase-function equations}
\usage{
computing_GMM_estimator(
  W,
  y,
  Zmat,
  weight_method = c("uniform", "minimax", "quasi-likelihood"),
  B = 100,
  t_grid_length = 1000
)
}
\arguments{
\item{W}{Numeric array of dimension \code{n x p x J} containing replicate
measurements of contaminated covariates.}

\item{y}{Numeric vector of length \code{n} containing the response.}

\item{Zmat}{Numeric matrix of dimension \code{n x q} containing error-free
covariates.}

\item{B}{Integer specifying the number of bootstrap replicates.}

\item{t_grid_length}{Integer specifying the length of optimal t for
computational intergration.}
}
\value{
A list with components:
\describe{
  \item{est}{Numeric matrix of GMM estimates; one column per weight choice.}
  \item{CV_boot}{Array of bootstrap covariance matrices used in GMM.}
}
}
\description{
Computes Generalized Method of Moments (GMM) estimators by combining
moment-corrected estimating equations and phase-function estimating
equations. The GMM weighting matrix is estimated via a cluster bootstrap
that accounts for uncertainty in estimating the measurement error
covariance.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Wrapper_output_functions.R
\name{vcov.eiv_mlr}
\alias{vcov.eiv_mlr}
\title{Variance-Covariance Matrix for eiv_mlr Objects}
\usage{
\method{vcov}{eiv_mlr}(object, ...)
}
\arguments{
\item{object}{An object of class \code{"eiv_mlr"}.}

\item{...}{Not used.}
}
\value{
A numeric variance-covariance matrix of the parameter estimates.
}
\description{
Variance-Covariance Matrix for eiv_mlr Objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Wrapper_output_functions.R
\name{print.eiv_mlr}
\alias{print.eiv_mlr}
\title{Print Method for eiv_mlr Objects}
\usage{
\method{print}{eiv_mlr}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"eiv_mlr"}.}

\item{...}{Not used.}
}
\value{
Invisibly returns the input object \code{x} of class \code{"eiv_mlr"}.
This function is called for its side effect of printing a summary of the model.
}
\description{
Displays a concise summary of an errors-in-variables linear model fit.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dietary_white_women.R
\docType{data}
\name{dietary_white_women}
\alias{dietary_white_women}
\title{dietary_white_women}
\format{
A data frame with 2 rows per individual (one per recall day).
\describe{
  \item{unit}{Integer unit identifier}
  \item{bmi}{Body Mass Index (kg/m^2)}
  \item{energy}{Total energy intake (kcal)}
  \item{protein}{Protein intake (g)}
  \item{fat}{Fat intake (g)}
  \item{age_in_month}{Age in months}
  \item{replicate}{Replicate id of each observation}
}
}
\source{
Processed from NHANES dietary recall data.
}
\usage{
dietary_white_women
}
\description{
A processed Dataset containing repeated 24-hour dietary recalls.
}
\details{
The variables \code{energy}, \code{protein}, and \code{fat}
are treated as error-prone covariates with two replicate
measurements per individual, and age_in_month is treated as error-free
covariate.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Wrapper_output_functions.R
\name{predict.eiv_mlr}
\alias{predict.eiv_mlr}
\title{Predictions from an Errors-in-Variables Linear Model}
\usage{
\method{predict}{eiv_mlr}(
  object,
  newdata = NULL,
  interval = c("none", "confidence"),
  level = 0.95,
  ...
)
}
\arguments{
\item{object}{An object of class \code{"eiv_mlr"}.}

\item{newdata}{Optional data frame containing covariates. If omitted,
predictions are returned for the training data.}

\item{interval}{Type of interval to compute: \code{"none"} or
\code{"confidence"}.}

\item{level}{Confidence level for intervals.}

\item{...}{Not used.}
}
\value{
A numeric vector of predictions, or a matrix with columns
  \code{fit}, \code{lwr}, and \code{upr} if intervals are requested.
}
\description{
Generates fitted values or confidence intervals for new observations
using plug-in estimates. Measurement error is not corrected for new data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estim_eq_correctedAndPhase.R
\name{estim_eq_correctedAndPhase}
\alias{estim_eq_correctedAndPhase}
\title{Combined estimating equations: moment correction and phase function}
\usage{
estim_eq_correctedAndPhase(
  Wbar,
  Zmat,
  y,
  S_y,
  C_y,
  b_in,
  estWeight,
  estSigmaU_bar,
  t
)
}
\arguments{
\item{Wbar}{Numeric matrix of dimension \code{n x p} containing averaged
error-prone covariates.}

\item{Zmat}{Numeric matrix of dimension \code{n x q} containing error-free
covariates (including an intercept if required).}

\item{y}{Numeric vector of length \code{n} containing the response.}

\item{S_y}{Numeric vector containing precomputed sine terms for the phase
function.}

\item{C_y}{Numeric vector containing precomputed cosine terms for the phase
function.}

\item{b_in}{Numeric vector of length \code{p + q} at which the estimating
equations are evaluated.}

\item{estWeight}{Numeric vector of weights used in the phase-function
component.}

\item{estSigmaU_bar}{Numeric array of dimension \code{n x p x p} containing
measurement error covariance estimates.}

\item{t}{Numeric vector of frequency values used in the phase function.}
}
\value{
Numeric vector containing the stacked estimating equations of length
  \code{2 * (p + q)}.
}
\description{
Constructs a stacked system of estimating equations combining
moment-corrected score equations and phase-function score equations.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PhaseGradient.R
\name{PhaseGradient}
\alias{PhaseGradient}
\title{Gradient of the phase-function estimating equations}
\usage{
PhaseGradient(beta, S_y, C_y, WZ, t, estWeight)
}
\arguments{
\item{beta}{Numeric vector of length \code{p + q} containing the regression
coefficients.}

\item{S_y}{Numeric vector containing \code{sum_j sin(t * Y_j)} evaluated on
the grid \code{t}.}

\item{C_y}{Numeric vector containing \code{sum_j cos(t * Y_j)} evaluated on
the grid \code{t}.}

\item{WZ}{Numeric matrix of dimension \code{n x (p + q)} containing all
covariates.}

\item{t}{Numeric vector giving the grid of frequency values.}

\item{estWeight}{Numeric vector of length \code{n} giving observation
weights.}
}
\value{
Numeric vector of length \code{p + q} containing the phase-function
  gradient evaluated at \code{beta}.
}
\description{
Computes the gradient of the phase-function estimating equations with
respect to the regression coefficients.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computing_corrected_estimator.R
\name{computing_corrected_estimator}
\alias{computing_corrected_estimator}
\title{Compute the corrected estimator via estimating equations}
\usage{
computing_corrected_estimator(y, Wbar, Zmat, estSigmaU_bar, beta_in = NULL)
}
\arguments{
\item{y}{Numeric vector of length \code{n}. Response variable.}

\item{Wbar}{Numeric matrix of dimension \code{n × p}. Error-prone
covariates at the averaged (replicate) level.}

\item{Zmat}{Numeric matrix of dimension \code{n × q}. Error-free
covariates (including intercept if applicable).}

\item{estSigmaU_bar}{Measurement error variance/covariance associated
with the averaged covariates. Can be \code{n × p × p} or \code{n × 1}.}

\item{beta_in}{Optional numeric vector of length \code{p + q}. Initial
value for the estimating equation solver.}
}
\value{
Numeric vector of length \code{p + q}, the corrected estimator.
}
\description{
This function computes the corrected estimator for a linear model
with measurement error in covariates by solving the corrected
estimating equations using a nonlinear root-finding algorithm.
}
\details{
The estimator is obtained by solving
\deqn{E_n(\beta) = 0}
where \code{E_n} is the corrected estimating equation implemented in
\code{estim_eq_corrected()}.
}
\seealso{
\code{\link{estim_eq_corrected}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Wrapper_output_functions.R
\name{coef.eiv_mlr}
\alias{coef.eiv_mlr}
\title{Extract Coefficients from an eiv_mlr Object}
\usage{
\method{coef}{eiv_mlr}(object, ...)
}
\arguments{
\item{object}{An object of class \code{"eiv_mlr"}.}

\item{...}{Not used.}
}
\value{
A named numeric vector of estimated regression coefficients.
}
\description{
Extract Coefficients from an eiv_mlr Object
}
